%-------------------------------------------------------------------------;
% Table 3: Additional predictability tests
% The code includes Monte-Carlo simulations for return and dividend growth 
% predictability for each period.
% Results are in Tab_3.
% Elapsed time: 185 seconds.  
%-------------------------------------------------------------------------;

clear all;
clc;

% '1' - Netherlands/U.K. period: 1629-1812;
% '2' - U.K. period; 1813-1870;
% '3' - U.S. early: 1871:1945;
% '4' - U.S. recent: 1945-2015;
% '34' - Combined U.S. period: 1871-2015;

%Choose nominal or real data;
a=2; %0 for nominal, 2 for real;

rng('default'); %Set the random number generator to default;

%-------------------;
% Netherlands/U.K.;
%-------------------;

filename='Data_Netherlands_UK_1602_1812.xls';
data=xlsread(filename);

%Create smoothed dividends (10-year average): nominal and real;
data(:,8)=filter(ones(10,1),1,data(:,3+a))./10;

%Determine time period: 1629-1812;
data_l=data((data(:,1)>=1629 & data(:,1)<=1812),:);
dates_1=data_l(:,1);

%Prices and dividends;
P=data_l(:,2+a); 
D=data_l(:,3+a);
D_10=data_l(:,8);

%Define variables. Returns always based on actual dividends;
RET_1=(P(2:end)+D(2:end))./(P(1:end-1))-1;

%Dividend growth up until including 1700 based on smoothed dividends, and
%on annual dividends thereafter. Make sure that DG is always based on either
%smoothed or annual dividends.
split=find(dates_1==1700);
DG_1=[D_10(2:split)./D_10(1:split-1)-1;D(split+1:end)./D(split:end-1)-1];

%Dividend-to-price ratio based on smoothed dividends up until including 
%1699, and on annual dividends thereafter; 
DP_1=[D_10(1:split-1)./P(1:split-1);D(split:end)./P(split:end)];

ret_1=log(1+RET_1);
dg_1=log(1+DG_1);
dp_1=log(DP_1);

%--------------------------------;
% Subperiod: from 1700 onwards;
%-------------------------------;

data_s=data((data(:,1)>=1700 & data(:,1)<=1812),:);
dates_s=data_s(:,1);

%Prices and dividends;
P=data_s(:,2+a); 
D=data_s(:,3+a);

%Define variables. 
RET_1_s=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_1_s=D(2:end)./D(1:end-1)-1;
DP_1_s=D(1:end)./P(1:end);

ret_1_s=log(1+RET_1_s);
dg_1_s=log(1+DG_1_s);
dp_1_s=log(DP_1_s);

%-------------------;
% U.K.;
%-------------------;

filename='Data_UK_1813_1870.xls';
data=xlsread(filename);

%Determine time period: 1813-1870;
data=data((data(:,1)>=1813 & data(:,1)<=1870),:);
dates_2=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a); 

%Define variables;
RET_2=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_2=(D(2:end))./(D(1:end-1))-1;
DP_2=(D(1:end))./(P(1:end));

ret_2=log(1+RET_2);
dg_2=log(1+DG_2);
dp_2=log(DP_2);

%----------------------;
% U.S. early 1871-1945
%----------------------;

filename='Data_US_1871_2015.xls';
data=xlsread(filename);

%Determine time period: 1871-2015;
data=data((data(:,1)>=1871 & data(:,1)<=1945),:);
dates_3=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a);

%Define variables;
RET_3=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_3=(D(2:end))./(D(1:end-1))-1;
DP_3=(D(1:end))./(P(1:end));

ret_3=log(1+RET_3);
dg_3=log(1+DG_3);
dp_3=log(DP_3);

%------------------------;
% U.S. recent: 1945-2015
%------------------------;

filename='Data_US_1871_2015.xls';
data=xlsread(filename);

%Determine time period: 1871-2015;
data=data((data(:,1)>=1945 & data(:,1)<=2015),:);
dates_4=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a);

%Define variables;
RET_4=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_4=(D(2:end))./(D(1:end-1))-1;
DP_4=(D(1:end))./(P(1:end));

ret_4=log(1+RET_4);
dg_4=log(1+DG_4);
dp_4=log(DP_4);

%----------------------;
% U.S. Full: 1871-2015
%----------------------;

filename='Data_US_1871_2015.xls';
data=xlsread(filename);

%Determine time period: 1871-2015;
data=data((data(:,1)>=1871 & data(:,1)<=2015),:);
dates_34=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a);

%Define variables;
RET_34=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_34=(D(2:end))./(D(1:end-1))-1;
DP_34=(D(1:end))./(P(1:end));

ret_34=log(1+RET_34);
dg_34=log(1+DG_34);
dp_34=log(DP_34);

%-------------------------------------------------------------------------;
% Loop to estimate parameters and perform Monte-Carlo for different sample
% periods;
%-------------------------------------------------------------------------;

Tab_3=zeros(21,6); %To store results;

for p=1:6 %Estimates for each column of Table 3;

    if p==1 %1629-1812;
        ret=ret_1;
        dg=dg_1;
        dp=dp_1;
    
        dp_f=dp_1(2:end);
        dp_l=dp_1(1:end-1);
    end;

    if p==2 %1813-1870;
        ret=ret_2;
        dg=dg_2;
        dp=dp_2;

        dp_f=dp_2(2:end);
        dp_l=dp_2(1:end-1);
    end;

    if p==3 %1871-1945;
        ret=ret_3;
        dg=dg_3;
        dp=dp_3;
    
        dp_f=dp_3(2:end);
        dp_l=dp_3(1:end-1);
    end;

    if p==4 %1945-2015;
        ret=ret_4;
        dg=dg_4;
        dp=dp_4;
    
        dp_f=dp_4(2:end);
        dp_l=dp_4(1:end-1);
    end;

    if p==5 %1700-2015;
        ret=[ret_1_s;ret_2;ret_34]; 
        dg=[dg_1_s;dg_2;dg_34]; 
        dp=[dp_1_s;dp_2;dp_34];
    
        dp_f=[dp_1_s(2:end);dp_2(2:end);dp_34(2:end)];
        dp_l=[dp_1_s(1:end-1);dp_2(1:end-1);dp_34(1:end-1)];    
    end;

    if p==6 %1629-2015;
        ret=[ret_1;ret_2;ret_34]; 
        dg=[dg_1;dg_2;dg_34]; 
        dp=[dp_1;dp_2;dp_34];
    
        dp_f=[dp_1(2:end);dp_2(2:end);dp_34(2:end)];
        dp_l=[dp_1(1:end-1);dp_2(1:end-1);dp_34(1:end-1)];
    end;

    mean_dp_org=mean(dp);
    std_dp_org=std(dp);

    %Rho;
    rho=exp(-mean_dp_org)/(1+exp(-mean_dp_org));

    %Predictive regressions;
    T=length(dp_l);

    %Returns;
    y=ret;
    x=[ones(T,1) dp_l];

    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
    beta_ret_org=b(2,1); %Save for simulations;
    pval_ret_org=2*(tcdf(-abs(tb(2,1)),length(y)-length(b)));
    res_ret=error; %Save for simulations;

    %Dividend growth;
    y=dg;
    x=[ones(T,1) dp_l];

    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
    beta_dg_org=b(2,1); %Save for simulations;
    pval_dg_org=2*(tcdf(-abs(tb(2,1)),length(y)-length(b)));
    res_dg=error; %Save for simulations;

    %Dividend-to-price ratio;
    y=dp_f;
    x=[ones(T,1) dp_l];

    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
    beta_dp_org=b(2,1); %Save for simulations;
    pval_dp_org=2*(tcdf(-abs(tb(2,1)),length(y)-length(b)));
    res_dp=error; %Save for simulations;

    %Implied estimates;
    beta_ret_impl=1+beta_dg_org-rho*beta_dp_org;
    beta_dg_impl=rho*beta_dp_org-1+beta_ret_org;
    beta_dp_impl=(1-beta_ret_org+beta_dg_org)/rho;

    %Correlation of residuals;
    correl_ret_dp=corr(res_ret,res_dp);
    correl_dg_dp=corr(res_dg,res_dp);

    %Collect the estimates;
    Tab_3(1:4,p)=[beta_ret_org;beta_ret_impl;correl_ret_dp;pval_ret_org];
    Tab_3(10:13,p)=[beta_dg_org;beta_dg_impl;correl_dg_dp;pval_dg_org];
    Tab_3(19:21,p)=[beta_dp_org;beta_dp_impl;pval_dp_org];

    %-----------------------------------;
    % Monte-Carlo simulations: returns;
    %-----------------------------------;

    %Null;
    beta_ret=0;
    beta_dp=beta_dp_org;
    beta_dg=rho*beta_dp-1+beta_ret;

    cov_dg_dp=cov(res_dg,res_dp);

    To=T+200-1;     % Sample length (use 200 periods as burn-in)
    No=50000;       % Number of simulated samples;

    %To store simulation results;
    beta_ret_sim=zeros(No,1); 
    beta_dg_sim=zeros(No,1);
    beta_dp_sim=zeros(No,1);
    DR_sim=zeros(No,1);

    for i=1:No
        %Joint distribution of errors;
        mu_error=zeros(To,2);
        error=mvnrnd(mu_error,cov_dg_dp);

        %Simulate returns, dividend growth, and the dividend-to-price ratio;
        ret=zeros(To,1);
        dg=zeros(To,1);
        dp=zeros(To,1);
        
        %Set initial value for dp;
        dp(1)=normrnd(0,std(res_dp)/(1-beta_dp));
        
        for t=2:To
            dp(t)=beta_dp*(dp(t-1))+error(t,2);
            dg(t)=beta_dg*(dp(t-1))+error(t,1);
            ret(t)=error(t,1)-rho*error(t,2);
        end;

        %Take out the burn-in period;
        ret=ret(201:end);
        dg=dg(201:end);
        dp=dp(200:end);

        %Returns;
        y=ret;  
        x=[ones(T-1,1) dp(1:end-1)];

        [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
        beta_ret_sim(i)=b(2,1);
    
        %Dividend growth;
        y=dg;   
        x=[ones(T-1,1) dp(1:end-1)];

        [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
        beta_dg_sim(i)=b(2,1);
    
        %Dividend-to-price ratio;
        y=dp(2:end);   
        x=[ones(T-1,1) dp(1:end-1)];

        [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
        beta_dp_sim(i)=b(2,1);
    
        %Preserve for long-run estimates;
        DR_sim(i)=beta_ret_sim(i)/(1-rho*beta_dp_sim(i)); 
    end;

    %P-values;
    a=find(beta_ret_sim>beta_ret_org);
    p_value_beta_ret_1=length(a)/length(beta_ret_sim);

    a=find(beta_dg_sim>beta_dg_org);
    p_value_beta_ret_2=length(a)/length(beta_dg_sim);

    Tab_3(5:6,p)=[p_value_beta_ret_1;p_value_beta_ret_2];

    %--------------------------------------------;
    % Monte-Carlo simulations: dividend growth;
    %--------------------------------------------;

    %Null;
    beta_dg=0;
    beta_dp=beta_dp_org;
    beta_ret=1+beta_dg-rho*beta_dp;

    cov_ret_dp=cov(res_ret,res_dp);

    To=T+200-1;     % Sample length (use 200 periods as burn-in)
    No=50000;       % Number of simulated samples;

    %To store simulation results;
    beta_ret_sim=zeros(No,1); 
    beta_dg_sim=zeros(No,1);
    beta_dp_sim=zeros(No,1);
    CF_sim=zeros(No,1);

    for i=1:No
        %Joint distribution of errors;
        mu_error=zeros(To,2);
        error=mvnrnd(mu_error,cov_ret_dp);

        %Simulate returns, dividend growth, and the dividend-to-price ratio;
        ret=zeros(To,1);
        dg=zeros(To,1);
        dp=zeros(To,1);
        
        %Set initial value for dp;
        dp(1)=normrnd(0,std(res_dp)/(1-beta_dp));
        
        for t=2:To
            dp(t)=beta_dp*(dp(t-1))+error(t,2);
            dg(t)=error(t,1)+rho*error(t,2);
            ret(t)=beta_ret*(dp(t-1))+error(t,1);
        end;

        %Take out the burn-in period;
        ret=ret(201:end);
        dg=dg(201:end);
        dp=dp(200:end);

        %Returns;
        y=ret;   
        x=[ones(T-1,1) dp(1:end-1)];

        [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
        beta_ret_sim(i)=b(2,1);
    
        %Dividend growth;
        y=dg;   
        x=[ones(T-1,1) dp(1:end-1)];

        [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
        beta_dg_sim(i)=b(2,1);
    
        %Dividend-to-price ratio;
        y=dp(2:end);   
        x=[ones(T-1,1) dp(1:end-1)];

        [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
        beta_dp_sim(i)=b(2,1);
    
        %Preserve for long-run estimates;
        CF_sim(i)=beta_dg_sim(i)/(1-rho*beta_dp_sim(i));
    end;

    %P-values;
    a=find(beta_dg_sim<beta_dg_org);
    p_value_beta_dg_1=length(a)/length(beta_dg_sim);

    a=find(beta_ret_sim<beta_ret_org);
    p_value_beta_dg_2=length(a)/length(beta_dg_sim);

    Tab_3(14:15,p)=[p_value_beta_dg_1;p_value_beta_dg_2];

    %-------------------------;
    %Long-run coefficients;
    %-------------------------;

    %Returns;
    DR=beta_ret_org/(1-rho*beta_dp_org);

    %Becasue we estimate parameters eq. by eq., we do not have covariances 
    %of the estimated parameters. We therefore implement delta method 
    %following Cochrane (2008) by relying on covariance of error terms;

    %Derivative wrt. beta_ret;
    A=1/(1-rho*beta_dp_org);

    %Derivative wrt. beta_dp;
    B=(rho*beta_ret_org)/((1-rho*beta_dp_org)^2);

    cov_ret_dp=cov(res_ret,res_dp);
    cov_ret_dp=cov_ret_dp(1,2);

    var_DR=A^2*var(res_ret)+B^2*var(res_dp)+2*A*B*cov_ret_dp;
    std_DR=(1/(sqrt(T-1)*std_dp_org))*sqrt(var_DR);
    t_DR=DR/std_DR;
    p_value_DR_delta=2*(tcdf(-abs(t_DR),(T-1)-4)); %We use 4 parameters;

    %P-values;
    a=find(DR_sim>DR);
    p_value_DR_sim=length(a)/length(DR_sim);

    Tab_3(7:9,p)=[DR;p_value_DR_delta;p_value_DR_sim];

    %Dividend growth rates;
    CF=beta_dg_org/(1-rho*beta_dp_org);

    %Derivative wrt. beta_dg;
    A=1/(1-rho*beta_dp_org);

    %Derivative wrt. beta_dp;
    B=(rho*beta_dg_org)/((1-rho*beta_dp_org)^2);

    cov_dg_dp=cov(res_dg,res_dp);
    cov_dg_dp=cov_dg_dp(1,2);

    var_CF=A^2*var(res_dg)+B^2*var(res_dp)+2*A*B*cov_dg_dp;
    std_CF=(1/(sqrt(T-1)*std_dp_org))*sqrt(var_CF);
    t_CF=CF/std_CF;
    p_value_CF_delta=2*(tcdf(-abs(t_CF),(T-1)-4)); %We use 4 parameters;

    %P-values;
    a=find(CF_sim<CF);
    p_value_CF_sim=length(a)/length(CF_sim);

    Tab_3(16:18,p)=[CF;p_value_CF_delta;p_value_CF_sim];
end;

Tab_3
