%-------------------------------------------------------------------------;
% Table 4: Long-horizon regressions, unweighted
% The code includes Monte-Carlo simulations for return and dividend growth 
% predictability for each period. 
% Results are in Tab_4.
% Elapsed time: 172 seconds.  
%-------------------------------------------------------------------------;

clear all;
clc;

% '1' - Netherlands/U.K. period: 1629-1812;
% '2' - U.K. period; 1813-1870;
% '3' - U.S. early: 1871:1945;
% '4' - U.S. recent: 1945-2015;
% '34' - Combined U.S. period: 1871-2015;

%Choose nominal or real data;
a=2; %0 for nominal, 2 for real;

rng('default'); %Set the random number generator to default;

%-------------------;
% Netherlands/U.K.;
%-------------------;

filename='Data_Netherlands_UK_1602_1812.xls';
data=xlsread(filename);

%Create smoothed dividends (10-year average): nominal and real;
data(:,8)=filter(ones(10,1),1,data(:,3+a))./10;

%Determine time period: 1629-1812;
data_l=data((data(:,1)>=1629 & data(:,1)<=1812),:);
dates_1=data_l(:,1);

%Prices and dividends;
P=data_l(:,2+a); 
D=data_l(:,3+a);
D_10=data_l(:,8);

%Define variables. Returns always based on actual dividends;
RET_1=(P(2:end)+D(2:end))./(P(1:end-1))-1;

%Dividend growth up until including 1700 based on smoothed dividends, and
%on annual dividends thereafter. Make sure that DG is always based on either
%smoothed or annual dividends.
split=find(dates_1==1700);
DG_1=[D_10(2:split)./D_10(1:split-1)-1;D(split+1:end)./D(split:end-1)-1];

%Dividend-to-price ratio based on smoothed dividends up until including 
%1699, and on annual dividends thereafter; 
DP_1=[D_10(1:split-1)./P(1:split-1);D(split:end)./P(split:end)];

ret_1=log(1+RET_1);
dg_1=log(1+DG_1);
dp_1=log(DP_1);

%--------------------------------;
%Subperiod: from 1700 onwards;
%-------------------------------;

data_s=data((data(:,1)>=1700 & data(:,1)<=1812),:);
dates_s=data_s(:,1);

%Prices and dividends;
P=data_s(:,2+a); 
D=data_s(:,3+a);

%Define variables. 
RET_1_s=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_1_s=D(2:end)./D(1:end-1)-1;
DP_1_s=D(1:end)./P(1:end);

ret_1_s=log(1+RET_1_s);
dg_1_s=log(1+DG_1_s);
dp_1_s=log(DP_1_s);

%-------------------;
% U.K.;
%-------------------;

filename='Data_UK_1813_1870.xls';
data=xlsread(filename);

%Determine time period: 1813-1870;
data=data((data(:,1)>=1813 & data(:,1)<=1870),:);
dates_2=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a); 

%Define variables;
RET_2=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_2=(D(2:end))./(D(1:end-1))-1;
DP_2=(D(1:end))./(P(1:end));

ret_2=log(1+RET_2);
dg_2=log(1+DG_2);
dp_2=log(DP_2);

%----------------------;
% U.S. early 1871-1945
%----------------------;

filename='Data_US_1871_2015.xls';
data=xlsread(filename);

%Determine time period: 1871-2015;
data=data((data(:,1)>=1871 & data(:,1)<=1945),:);
dates_3=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a);

%Define variables;
RET_3=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_3=(D(2:end))./(D(1:end-1))-1;
DP_3=(D(1:end))./(P(1:end));

ret_3=log(1+RET_3);
dg_3=log(1+DG_3);
dp_3=log(DP_3);

%------------------------;
% U.S. recent: 1945-2015
%------------------------;

filename='Data_US_1871_2015.xls';
data=xlsread(filename);

%Determine time period: 1871-2015;
data=data((data(:,1)>=1945 & data(:,1)<=2015),:);
dates_4=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a);

%Define variables;
RET_4=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_4=(D(2:end))./(D(1:end-1))-1;
DP_4=(D(1:end))./(P(1:end));

ret_4=log(1+RET_4);
dg_4=log(1+DG_4);
dp_4=log(DP_4);

%----------------------;
% U.S. Full: 1871-2015
%----------------------;

filename='Data_US_1871_2015.xls';
data=xlsread(filename);

%Determine time period: 1871-2015;
data=data((data(:,1)>=1871 & data(:,1)<=2015),:);
dates_34=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a);

%Define variables;
RET_34=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_34=(D(2:end))./(D(1:end-1))-1;
DP_34=(D(1:end))./(P(1:end));

ret_34=log(1+RET_34);
dg_34=log(1+DG_34);
dp_34=log(DP_34);

%-------------------------------------------------------------------------;
% Loop to estimate parameters and perform Monte-Carlo for different sample
% periods;
%-------------------------------------------------------------------------;

Tab_4=zeros(36,4); %To store results;

for p=1:4 %For each column of Table 4;

%------------------------------------------------;
% Estimates 1-year horizon;
%------------------------------------------------;

if p==1 %1629-1945;
    ret=[ret_1;ret_2;ret_3]; 
    dg=[dg_1;dg_2;dg_3]; 
    dp=[dp_1;dp_2;dp_3];
    
    dp_f=[dp_1(2:end);dp_2(2:end);dp_3(2:end)];
    dp_l=[dp_1(1:end-1);dp_2(1:end-1);dp_3(1:end-1)];
end;

if p==2 %1945-2015;
    ret=ret_4;
    dg=dg_4;
    dp=dp_4;
    
    dp_f=dp_4(2:end);
    dp_l=dp_4(1:end-1);
end;

if p==3 %1700-2015;
    ret=[ret_1_s;ret_2;ret_34]; 
    dg=[dg_1_s;dg_2;dg_34]; 
    dp=[dp_1_s;dp_2;dp_34];
    
    dp_f=[dp_1_s(2:end);dp_2(2:end);dp_34(2:end)];
    dp_l=[dp_1_s(1:end-1);dp_2(1:end-1);dp_34(1:end-1)];
end;

if p==4 %1629-2015;
    ret=[ret_1;ret_2;ret_34]; 
    dg=[dg_1;dg_2;dg_34]; 
    dp=[dp_1;dp_2;dp_34];
    
    dp_f=[dp_1(2:end);dp_2(2:end);dp_34(2:end)];
    dp_l=[dp_1(1:end-1);dp_2(1:end-1);dp_34(1:end-1)];
end;

%Rho;
mean_dp_org=mean(dp);
rho=exp(-mean_dp_org)/(1+exp(-mean_dp_org));

%Predictive regressions;
T=length(dp_l);

%Returns;
y=ret;
x=[ones(T,1) dp_l];

[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
beta_ret_org=b(2,1); %Save for simulations;
tb_ret_org=tb(2,1);
R2_ret=R2;
res_ret=error; %Save for simulations;

%Dividend growth;
y=dg;
x=[ones(T,1) dp_l];

[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
beta_dg_org=b(2,1); %Save for simulations;
tb_dg_org=tb(2,1); 
R2_dg=R2;
res_dg=error; %Save for simulations;

%Dividend-to-price ratio;
y=dp_f;
x=[ones(T,1) dp_l];

[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
beta_dp_org=b(2,1); %Save for simulations;
res_dp=error; %Save for simulations;

Tab_4(1:2,p)=[beta_ret_org;tb_ret_org];
Tab_4(4,p)=R2_ret;
Tab_4(19:20,p)=[beta_dg_org;tb_dg_org];
Tab_4(22,p)=R2_dg;

%------------------------------------------------;
% Estimates 3-year horizon;
%------------------------------------------------;

h=3;

%Sum returns over h periods;
ret_h_1=filter(ones(h,1),1,ret_1);
ret_h_1_s=filter(ones(h,1),1,ret_1_s);
ret_h_2=filter(ones(h,1),1,ret_2);
ret_h_3=filter(ones(h,1),1,ret_3);
ret_h_4=filter(ones(h,1),1,ret_4);
ret_h_34=filter(ones(h,1),1,ret_34);

%Sum dividend growth over h periods;
dg_h_1=filter(ones(h,1),1,dg_1);
dg_h_1_s=filter(ones(h,1),1,dg_1_s);
dg_h_2=filter(ones(h,1),1,dg_2);
dg_h_3=filter(ones(h,1),1,dg_3);
dg_h_4=filter(ones(h,1),1,dg_4);
dg_h_34=filter(ones(h,1),1,dg_34);

if p==1 %1629-1945;
ret_h=[ret_h_1(h:end);ret_h_2(h:end);ret_h_3(h:end)];
dg_h=[dg_h_1(h:end);dg_h_2(h:end);dg_h_3(h:end)];
dp_h=[dp_1(1:end-h);dp_2(1:end-h);dp_3(1:end-h)];
end;

if p==2 %1945-2015;
ret_h=ret_h_4(h:end);
dg_h=dg_h_4(h:end);
dp_h=dp_4(1:end-h);
end;

if p==3 %1700-2015;
ret_h=[ret_h_1_s(h:end);ret_h_2(h:end);ret_h_34(h:end)];
dg_h=[dg_h_1_s(h:end);dg_h_2(h:end);dg_h_34(h:end)];
dp_h=[dp_1_s(1:end-h);dp_2(1:end-h);dp_34(1:end-h)];
end;

if p==4 %1629-2015;
ret_h=[ret_h_1(h:end);ret_h_2(h:end);ret_h_34(h:end)];
dg_h=[dg_h_1(h:end);dg_h_2(h:end);dg_h_34(h:end)];
dp_h=[dp_1(1:end-h);dp_2(1:end-h);dp_34(1:end-h)];
end;

%Returns;
y=ret_h;
x=[ones(length(dp_h),1) dp_h];
[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,h);
beta_ret_org_3y=b(2,1); %Save for simulations;
tb_ret_org_3y=tb(2,1);
R2_ret_3y=R2;

%Dividend growth;
y=dg_h;
x=[ones(length(dp_h),1) dp_h];
[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,h);
beta_dg_org_3y=b(2,1); %Save for simulations; 
tb_dg_org_3y=tb(2,1);
R2_dg_3y=R2;

%Non-overlapping;
tb_ret_org_no_h=zeros(0,h); %To store results;
tb_dg_org_no_h=zeros(0,h);

for i=1:h
    %Returns;
    y=ret_h(i:h:end);
    x=[ones(length(dp_h(i:h:end)),1) dp_h(i:h:end)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
    tb_ret_org_no_h(i)=tb(2,1);

    %Dividend growth;
    y=dg_h(i:h:end);
    x=[ones(length(dp_h(i:h:end)),1) dp_h(i:h:end)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
    tb_dg_org_no_h(i)=tb(2,1);
end;

tb_ret_org_no_3y=mean(tb_ret_org_no_h);
tb_dg_org_no_3y=mean(tb_dg_org_no_h);

Tab_4(5:7,p)=[beta_ret_org_3y;tb_ret_org_3y;tb_ret_org_no_3y];
Tab_4(11,p)=R2_ret_3y;
Tab_4(23:25,p)=[beta_dg_org_3y;tb_dg_org_3y;tb_dg_org_no_3y];
Tab_4(29,p)=R2_dg_3y;

%------------------------------------------------;
% Estimates 5-year horizon;
%------------------------------------------------;

h=5;

%Sum returns over h periods;
ret_h_1=filter(ones(h,1),1,ret_1);
ret_h_1_s=filter(ones(h,1),1,ret_1_s);
ret_h_2=filter(ones(h,1),1,ret_2);
ret_h_3=filter(ones(h,1),1,ret_3);
ret_h_4=filter(ones(h,1),1,ret_4);
ret_h_34=filter(ones(h,1),1,ret_34);

%Sum dividend growth over h periods;
dg_h_1=filter(ones(h,1),1,dg_1);
dg_h_1_s=filter(ones(h,1),1,dg_1_s);
dg_h_2=filter(ones(h,1),1,dg_2);
dg_h_3=filter(ones(h,1),1,dg_3);
dg_h_4=filter(ones(h,1),1,dg_4);
dg_h_34=filter(ones(h,1),1,dg_34);

if p==1 %1629-1945;
ret_h=[ret_h_1(h:end);ret_h_2(h:end);ret_h_3(h:end)];
dg_h=[dg_h_1(h:end);dg_h_2(h:end);dg_h_3(h:end)];
dp_h=[dp_1(1:end-h);dp_2(1:end-h);dp_3(1:end-h)];
end;

if p==2 %1945-2015;
ret_h=ret_h_4(h:end);
dg_h=dg_h_4(h:end);
dp_h=dp_4(1:end-h);
end;

if p==3 %1700-2015;
ret_h=[ret_h_1_s(h:end);ret_h_2(h:end);ret_h_34(h:end)];
dg_h=[dg_h_1_s(h:end);dg_h_2(h:end);dg_h_34(h:end)];
dp_h=[dp_1_s(1:end-h);dp_2(1:end-h);dp_34(1:end-h)];
end;

if p==4 %1629-2015;
ret_h=[ret_h_1(h:end);ret_h_2(h:end);ret_h_34(h:end)];
dg_h=[dg_h_1(h:end);dg_h_2(h:end);dg_h_34(h:end)];
dp_h=[dp_1(1:end-h);dp_2(1:end-h);dp_34(1:end-h)];
end;

%Returns;
y=ret_h;
x=[ones(length(dp_h),1) dp_h];
[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,h);
beta_ret_org_5y=b(2,1); %Save for simulations;
tb_ret_org_5y=tb(2,1);
R2_ret_5y=R2;

%Dividend growth;
y=dg_h;
x=[ones(length(dp_h),1) dp_h];
[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,h);
beta_dg_org_5y=b(2,1); %Save for simulations; 
tb_dg_org_5y=tb(2,1);
R2_dg_5y=R2;

%Non-overlapping;
tb_ret_org_no_h=zeros(0,h); %To store results;
tb_dg_org_no_h=zeros(0,h);

for i=1:h
    %Returns;
    y=ret_h(i:h:end);
    x=[ones(length(dp_h(i:h:end)),1) dp_h(i:h:end)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1); 
    tb_ret_org_no_h(i)=tb(2,1);

    %Dividend growth;
    y=dg_h(i:h:end);
    x=[ones(length(dp_h(i:h:end)),1) dp_h(i:h:end)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
    tb_dg_org_no_h(i)=tb(2,1);
end;

tb_ret_org_no_5y=mean(tb_ret_org_no_h);
tb_dg_org_no_5y=mean(tb_dg_org_no_h);

Tab_4(12:14,p)=[beta_ret_org_5y;tb_ret_org_5y;tb_ret_org_no_5y];
Tab_4(18,p)=R2_ret_5y;
Tab_4(30:32,p)=[beta_dg_org_5y;tb_dg_org_5y;tb_dg_org_no_5y];
Tab_4(36,p)=R2_dg_5y;

%---------------------------------------;
% VAR implied parameters, unweighted;
%---------------------------------------;

beta_ret_impl_1y=beta_ret_org;
beta_ret_impl_3y=beta_ret_org*(1-beta_dp_org^3)/(1-beta_dp_org);
beta_ret_impl_5y=beta_ret_org*(1-beta_dp_org^5)/(1-beta_dp_org);

beta_dg_impl_1y=beta_dg_org;
beta_dg_impl_3y=beta_dg_org*(1-beta_dp_org^3)/(1-beta_dp_org);
beta_dg_impl_5y=beta_dg_org*(1-beta_dp_org^5)/(1-beta_dp_org);

Tab_4(9,p)=beta_ret_impl_3y;
Tab_4(16,p)=beta_ret_impl_5y;
Tab_4(27,p)=beta_dg_impl_3y;
Tab_4(34,p)=beta_dg_impl_5y;

%-----------------------------------;
% Monte-Carlo simulations - returns;
%-----------------------------------;

beta_ret=0;
beta_dp=beta_dp_org;
beta_dg=rho*beta_dp-1+beta_ret;

%Simulations;

cov_dg_dp=cov(res_dg,res_dp);

To=T+200-1;     % Sample length (use 200 periods as burn-in)
No=50000;       % Number of simulated samples;

%To store simulation results;
beta_ret_sim=zeros(No,1); 
beta_dp_sim=zeros(No,1);
beta_ret_sim_3y=zeros(No,1);
beta_ret_sim_5y=zeros(No,1);
beta_ret_sim_impl_1y=zeros(No,1);
beta_ret_sim_impl_3y=zeros(No,1);
beta_ret_sim_impl_5y=zeros(No,1);

for i=1:No

    %Joint distribution of errors;
    mu_error=zeros(To,2);
    error=mvnrnd(mu_error,cov_dg_dp);

    %Simulate returns, dividend growth, and the dividend-to-price ratio;
    ret=zeros(To,1);
    dg=zeros(To,1);
    dp=zeros(To,1);
    
    %Set initial value for dp;
    dp(1)=normrnd(0,std(res_dp)/(1-beta_dp));
    
    for t=2:To
        dp(t)=beta_dp*(dp(t-1))+error(t,2);
        dg(t)=beta_dg*(dp(t-1))+error(t,1);
        ret(t)=error(t,1)-rho*error(t,2);
    end;

    %Take out the burn-in period;
    ret=ret(201:end);
    dg=dg(201:end);
    dp=dp(200:end);

    %Returns;
    y=ret;  
    x=[ones(T-1,1) dp(1:end-1)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_ret_sim(i)=b(2,1);

    %Dividend-to-price ratio;
    y=dp(2:end);   
    x=[ones(T-1,1) dp(1:end-1)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_dp_sim(i)=b(2,1);

    %Longer-horizon direct estimates;

    %3-years;
    h=3;
    ret_3y=filter(ones(h,1),1,ret);
    ret_3y=ret_3y(h:end);

    %Returns;
    y=ret_3y;
    x=[ones(length(ret_3y),1) dp(1:end-h)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_ret_sim_3y(i)=b(2,1); 

    %5-years;
    h=5; 
    ret_5y=filter(ones(h,1),1,ret);
    ret_5y=ret_5y(h:end);

    %Returns;
    y=ret_5y;
    x=[ones(length(ret_5y),1) dp(1:end-h)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_ret_sim_5y(i)=b(2,1); 

    %VAR implied estimates;
    beta_ret_sim_impl_1y(i)=beta_ret_sim(i);
    beta_ret_sim_impl_3y(i)=beta_ret_sim(i)*(1-beta_dp_sim(i)^3)/(1-beta_dp_sim(i));
    beta_ret_sim_impl_5y(i)=beta_ret_sim(i)*(1-beta_dp_sim(i)^5)/(1-beta_dp_sim(i));
end;

%P-values: Direct estimates;
a=find(beta_ret_sim>beta_ret_org);
p_value_beta_ret_1y=length(a)/length(beta_ret_sim);

a=find(beta_ret_sim_3y>beta_ret_org_3y);
p_value_beta_ret_3y=length(a)/length(beta_ret_sim_3y);

a=find(beta_ret_sim_5y>beta_ret_org_5y);
p_value_beta_ret_5y=length(a)/length(beta_ret_sim_5y);

Tab_4(3,p)=p_value_beta_ret_1y;
Tab_4(8,p)=p_value_beta_ret_3y;
Tab_4(15,p)=p_value_beta_ret_5y;

%P-values: Implied estimates;
a=find(beta_ret_sim_impl_1y>beta_ret_impl_1y);
p_value_beta_ret_impl_1y=length(a)/length(beta_ret_sim_impl_1y);

a=find(beta_ret_sim_impl_3y>beta_ret_impl_3y);
p_value_beta_ret_impl_3y=length(a)/length(beta_ret_sim_impl_3y);

a=find(beta_ret_sim_impl_5y>beta_ret_impl_5y);
p_value_beta_ret_impl_5y=length(a)/length(beta_ret_sim_impl_5y);

Tab_4(10,p)=p_value_beta_ret_impl_3y;
Tab_4(17,p)=p_value_beta_ret_impl_5y;

%--------------------------------------------;
% Monte-Carlo simulations - dividend growth;
%--------------------------------------------;

beta_dg=0;
beta_dp=beta_dp_org;
beta_ret=1+beta_dg-rho*beta_dp;

cov_ret_dp=cov(res_ret,res_dp);

To=T+200-1;     % Sample length (use 200 periods as burn-in)
No=50000;       % Number of simulated samples;

%To store simulation results;
beta_dg_sim=zeros(No,1);
beta_dp_sim=zeros(No,1);
beta_dg_sim_3y=zeros(No,1);
beta_dg_sim_5y=zeros(No,1);
beta_dg_sim_impl_1y=zeros(No,1);
beta_dg_sim_impl_3y=zeros(No,1);
beta_dg_sim_impl_5y=zeros(No,1);

for i=1:No

    %Joint distribution of errors;
    mu_error=zeros(To,2);
    error=mvnrnd(mu_error,cov_ret_dp);

    %Simulate returns, dividend growth, and the dividend-to-price ratio;
    ret=zeros(To,1);
    dg=zeros(To,1);
    dp=zeros(To,1);

    %Set initial value for dp;
    dp(1)=normrnd(0,std(res_dp)/(1-beta_dp));
        
    for t=2:To
        dp(t)=beta_dp*(dp(t-1))+error(t,2);
        dg(t)=error(t,1)+rho*error(t,2);
        ret(t)=beta_ret*(dp(t-1))+error(t,1);
    end;

    %Take out the burn-in period;
    ret=ret(201:end);
    dg=dg(201:end);
    dp=dp(200:end);

    %Dividend growth;
    y=dg;   
    x=[ones(T-1,1) dp(1:end-1)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_dg_sim(i)=b(2,1);

    %Dividend-to-price ratio;
    y=dp(2:end);   
    x=[ones(T-1,1) dp(1:end-1)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_dp_sim(i)=b(2,1);

    %Longer-horizon direct estimates;

    %3-years;
    h=3;
    dg_3y=filter(ones(h,1),1,dg);
    dg_3y=dg_3y(h:end);

    %Dividend growth;
    y=dg_3y;
    x=[ones(length(ret_3y),1) dp(1:end-h)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_dg_sim_3y(i)=b(2,1); 

    %5-years;
    h=5;
    dg_5y=filter(ones(h,1),1,dg);
    dg_5y=dg_5y(h:end);

    %Dividend growth;
    y=dg_5y;
    x=[ones(length(ret_5y),1) dp(1:end-h)];
    [b,b_std,tb,R2,AR2,error]=olsnw(y,x,0,0); %OLS errors to speed up;
    beta_dg_sim_5y(i)=b(2,1); 

    %VAR implied estimates;
    beta_dg_sim_impl_1y(i)=beta_dg_sim(i);
    beta_dg_sim_impl_3y(i)=beta_dg_sim(i)*(1-beta_dp_sim(i)^3)/(1-beta_dp_sim(i));
    beta_dg_sim_impl_5y(i)=beta_dg_sim(i)*(1-beta_dp_sim(i)^5)/(1-beta_dp_sim(i));
end;

%P-values: Direct estimates;
a=find(beta_dg_sim<beta_dg_org);
p_value_beta_dg_1y=length(a)/length(beta_dg_sim);

a=find(beta_dg_sim_3y<beta_dg_org_3y);
p_value_beta_dg_3y=length(a)/length(beta_dg_sim_3y);

a=find(beta_dg_sim_5y<beta_dg_org_5y);
p_value_beta_dg_5y=length(a)/length(beta_dg_sim_5y);

Tab_4(21,p)=p_value_beta_dg_1y;
Tab_4(26,p)=p_value_beta_dg_3y;
Tab_4(33,p)=p_value_beta_dg_5y;

%P-values: Implied estimates;
a=find(beta_dg_sim_impl_1y<beta_dg_impl_1y);
p_value_beta_dg_impl_1y=length(a)/length(beta_dg_sim_impl_1y);

a=find(beta_dg_sim_impl_3y<beta_dg_impl_3y);
p_value_beta_dg_impl_3y=length(a)/length(beta_dg_sim_impl_3y);

a=find(beta_dg_sim_impl_5y<beta_dg_impl_5y);
p_value_beta_dg_impl_5y=length(a)/length(beta_dg_sim_impl_5y);

Tab_4(28,p)=p_value_beta_dg_impl_3y;
Tab_4(35,p)=p_value_beta_dg_impl_5y;
end;

Tab_4

