%-------------------------------------------------------------------------;
% Table 5: Out-of-sample forecasts
% Number of simulations set to 1,000. In the paper, results are based on 
% 50,000 simulations.
% Results are in Tab_5.
% Elapsed time: 93 seconds
%-------------------------------------------------------------------------;

clear all;
clc;

% '1' - Netherlands/U.K. period: 1629-1812;
% '2' - U.K. period; 1813-1870;
% '3' - U.S. early: 1871:1945;
% '4' - U.S. recent: 1945-2015;
% '34' - Combined U.S. period: 1871-2015;

%Choose nominal or real data;
a=2; %0 for nominal, 2 for real;

rng('default'); %Set the random number generator to default;

%-------------------;
% Netherlands/U.K.;
%-------------------;

filename='Data_Netherlands_UK_1602_1812.xls';
data=xlsread(filename);

%Create smoothed dividends (10-year average): nominal and real;
data(:,8)=filter(ones(10,1),1,data(:,3+a))./10;

%Determine time period: 1629-1812;
data_l=data((data(:,1)>=1629 & data(:,1)<=1812),:);
dates_1=data_l(:,1);

%Prices and dividends;
P=data_l(:,2+a); 
D=data_l(:,3+a);
D_10=data_l(:,8);

%Define variables. Returns always based on actual dividends;
RET_1=(P(2:end)+D(2:end))./(P(1:end-1))-1;

%Dividend growth up until including 1700 based on smoothed dividends, and
%on annual dividends thereafter. Make sure that DG is always based on either
%smoothed or annual dividends.
split=find(dates_1==1700);
DG_1=[D_10(2:split)./D_10(1:split-1)-1;D(split+1:end)./D(split:end-1)-1];

%Dividend-to-price ratio based on smoothed dividends up until including 
%1699, and on annual dividends thereafter; 
DP_1=[D_10(1:split-1)./P(1:split-1);D(split:end)./P(split:end)];

ret_1=log(1+RET_1);
dg_1=log(1+DG_1);
dp_1=log(DP_1);

%--------------------------------;
%Subperiod: from 1700 onwards;
%-------------------------------;

data_s=data((data(:,1)>=1700 & data(:,1)<=1812),:);
dates_s=data_s(:,1);

%Prices and dividends;
P=data_s(:,2+a); 
D=data_s(:,3+a);

%Define variables. 
RET_1_s=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_1_s=D(2:end)./D(1:end-1)-1;
DP_1_s=D(1:end)./P(1:end);

ret_1_s=log(1+RET_1_s);
dg_1_s=log(1+DG_1_s);
dp_1_s=log(DP_1_s);

%-------------------;
% U.K.;
%-------------------;

filename='Data_UK_1813_1870.xls';
data=xlsread(filename);

%Determine time period: 1813-1870;
data=data((data(:,1)>=1813 & data(:,1)<=1870),:);
dates_2=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a); 

%Define variables;
RET_2=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_2=(D(2:end))./(D(1:end-1))-1;
DP_2=(D(1:end))./(P(1:end));

ret_2=log(1+RET_2);
dg_2=log(1+DG_2);
dp_2=log(DP_2);

%----------------------;
% U.S. Full: 1871-2015
%----------------------;

filename='Data_US_1871_2015.xls';
data=xlsread(filename);

%Determine time period: 1871-2015;
data=data((data(:,1)>=1871 & data(:,1)<=2015),:);
dates_34=data(:,1);

%Prices and dividends;
P=data(:,2+a); 
D=data(:,3+a);

%Define variables;
RET_34=(P(2:end)+D(2:end))./(P(1:end-1))-1;
DG_34=(D(2:end))./(D(1:end-1))-1;
DP_34=(D(1:end))./(P(1:end));

ret_34=log(1+RET_34);
dg_34=log(1+DG_34);
dp_34=log(DP_34);

%----------------------------------------------------;
% Out-of-sample predictions;
%----------------------------------------------------;

%To store results;
R2_OOS=zeros(1,5); %Unconstrained ROOS;
R2_OOS_constr_one=zeros(1,5); %Non-negative slope;
R2_OOS_constr_two=zeros(1,5); %Non-negative forecast;
WC=zeros(1,5); %West and Clark (2007) for the unconstraint case;

for h=1:2:5 %For 1-,3- and 5-year forecasts;

    %Sums returns over h years for each period separately;     
    ret_h_1=filter(ones(h,1),1,ret_1);
    ret_h_2=filter(ones(h,1),1,ret_2);
    ret_h_34=filter(ones(h,1),1,ret_34);

    %Append the data;
    ret_h=[ret_h_1;ret_h_2(h:end);ret_h_34(h:end)];
    dp=[dp_1(1:end-h);dp_2(1:end-h);dp_34(1:end)]; 

    T = size(dp,1);

    a=72; %Starting point for expanding windows. First prediction based on 
    %the value of dp in 1700;

    %To store forecasts;
    ret_f=zeros(T-h,1); %Forecasts based on a constant and dp;
    ret_f_constr_one=zeros(T-h,1); %Non-negative slope;
    ret_f_constr_two=zeros(T-h,1); %Non-negative forecast;
    ret_fc=zeros(T-h,1); %Forecasts based on a constant;

    for i=a:T-h
        [b,b_std,tb,R2,AR2,error]=olsnw(ret_h(h:i-1),[ones(i-h,1) dp(1:i-h)],1,1);
        ret_f(i+h-1)=b(1,1)+b(2,1)*dp(i);
        if b(2,1)>=0 
            ret_f_constr_one(i+h-1)=b(1,1)+b(2,1)*dp(i);
        else
            ret_f_constr_one(i+h-1)=mean(ret_h(h:i-1));
        end;
        if ret_f(i+h-1)>=0
            ret_f_constr_two(i+h-1)=ret_f(i+h-1);
        else
            ret_f_constr_two(i+h-1)=0;
        end;
        ret_fc(i+h-1)=mean(ret_h(h:i-1));
    end;

    %Actual returns and forecasts;
    ret_a=ret_h(a+h-1:end);
    ret_f=ret_f(a+h-1:end); 
    ret_f_constr_one=ret_f_constr_one(a+h-1:end);
    ret_f_constr_two=ret_f_constr_two(a+h-1:end);
    ret_fc=ret_fc(a+h-1:end);

    %Forecasting errors;
    err_f=ret_a-ret_f;
    err_f_constr_one=ret_a-ret_f_constr_one;
    err_f_constr_two=ret_a-ret_f_constr_two;
    err_fc=ret_a-ret_fc;

    %Out-of-sample R-square;
    R2_OOS(h)=1.-(err_f'*err_f)./(err_fc'*err_fc);
    R2_OOS_constr_one(h)=1.-(err_f_constr_one'*err_f_constr_one)./(err_fc'*err_fc);
    R2_OOS_constr_two(h)=1.-(err_f_constr_two'*err_f_constr_two)./(err_fc'*err_fc);

    %West and Clark (2007);
    WC_adj=err_fc.^2-(err_f.^2-(err_f-err_fc).^2);
    [b,b_std,tb,R2,AR2,error]=olsnw(WC_adj,ones(size(WC_adj,1),1),1,h);
    WC(h)=tb;
end;

%Collect results;
Tab_5(1,1:3)=[R2_OOS(1,1);R2_OOS_constr_one(1,1);R2_OOS_constr_two(1,1)];
Tab_5(2,1:3)=[WC(1,1) NaN NaN];
Tab_5(4,1:3)=[R2_OOS(1,3);R2_OOS_constr_one(1,3);R2_OOS_constr_two(1,3)];
Tab_5(5,1:3)=[WC(1,3) NaN NaN];
Tab_5(7,1:3)=[R2_OOS(1,5);R2_OOS_constr_one(1,5);R2_OOS_constr_two(1,5)];
Tab_5(8,1:3)=[WC(1,5) NaN NaN];

%Save for simulations;
R2_OOS_1y=R2_OOS(1,1);
R2_OOS_3y=R2_OOS(1,3);
R2_OOS_5y=R2_OOS(1,5);

%-----------------------------------------;
% Many different sample splits (20-80%)
%-----------------------------------------;

%To store results;
R2_OOS_a=zeros(1,5); %Average unconstrained ROOS;
R2_OOS_constr_one_a=zeros(1,5); %Non-negative slope;
R2_OOS_constr_two_a=zeros(1,5); %Non-negative forecast;
WC_a=zeros(1,5); %Average West and Clark (2007) for unconstrained case;

for h=1:2:5 %For 1-,3- and 5-year forecasts;
    
    %Sums returns over h years for each period separately;    
    ret_h_1=filter(ones(h,1),1,ret_1);
    ret_h_2=filter(ones(h,1),1,ret_2);
    ret_h_34=filter(ones(h,1),1,ret_34);

    %Append the data;
    ret_h=[ret_h_1;ret_h_2(h:end);ret_h_34(h:end)];
    dp=[dp_1(1:end-h);dp_2(1:end-h);dp_34(1:end)]; 

    T = size(dp,1);
    
    %Determine min/max for the sample splits;
    d=round(size(ret_h,1)*0.2); 
    f=round(size(ret_h,1)*0.8); 

    %To store results;
    R2_OOS=zeros(f,1);
    R2_OOS_constr_one=zeros(f,1);
    R2_OOS_constr_two=zeros(f,1);
    WC=zeros(f,1);
    
    for a=d:f
        %To store forecasts;
        ret_f=zeros(T-h,1); %Forecasts using a constant and dp;
        ret_f_constr_one=zeros(T-h,1); %Constraint: non-negative slope;
        ret_f_constr_two=zeros(T-h,1); %Constraint: non-negative forecast;
        ret_fc=zeros(T-h,1); %Forecasts based on a constant;

        %Expanding windows;
        for i=a:T-h
            [b,b_std,tb,R2,AR2,error]=olsnw(ret_h(h:i-1),[ones(i-h,1) dp(1:i-h)],1,1);
            ret_f(i+h-1)=b(1,1)+b(2,1)*dp(i);
            if b(2,1)>=0 
                ret_f_constr_one(i+h-1)=b(1,1)+b(2,1)*dp(i);
            else
                ret_f_constr_one(i+h-1)=mean(ret_h(h:i-1));
            end;
            if ret_f(i+h-1)>=0
                ret_f_constr_two(i+h-1)=ret_f(i+h-1);
            else
                ret_f_constr_two(i+h-1)=0;
            end;
            ret_fc(i+h-1)=mean(ret_h(h:i-1));
        end;

        %Actual returns and forecasts;
        ret_a=ret_h(a+h-1:end);
        ret_f=ret_f(a+h-1:end); 
        ret_f_constr_one=ret_f_constr_one(a+h-1:end);
        ret_f_constr_two=ret_f_constr_two(a+h-1:end);
        ret_fc=ret_fc(a+h-1:end);

        %Forecasting errors;
        err_f=ret_a-ret_f;
        err_f_constr_one=ret_a-ret_f_constr_one;
        err_f_constr_two=ret_a-ret_f_constr_two;
        err_fc=ret_a-ret_fc;

        %Out-of-sample R-square;
        R2_OOS(a)=1.-(err_f'*err_f)./(err_fc'*err_fc);
        R2_OOS_constr_one(a)=1.-(err_f_constr_one'*err_f_constr_one)./(err_fc'*err_fc);
        R2_OOS_constr_two(a)=1.-(err_f_constr_two'*err_f_constr_two)./(err_fc'*err_fc);
        
        %West and Clark (2007);
        WC_adj=err_fc.^2-(err_f.^2-(err_f-err_fc).^2);
        [b,b_std,tb,R2,AR2,error]=olsnw(WC_adj,ones(size(WC_adj,1),1),1,h);
        WC(a)=tb;
    end;

    R2_OOS_a(h)=mean(R2_OOS(d:f));
    R2_OOS_constr_one_a(h)=mean(R2_OOS_constr_one(d:f));
    R2_OOS_constr_two_a(h)=mean(R2_OOS_constr_two(d:f));
    WC_a(h)=mean(WC(d:f));
end;

%Collect results;
Tab_5(1,4:6)=[R2_OOS_a(1,1);R2_OOS_constr_one_a(1,1);R2_OOS_constr_two_a(1,1)];
Tab_5(2,4:6)=[WC_a(1,1) NaN NaN];
Tab_5(4,4:6)=[R2_OOS_a(1,3);R2_OOS_constr_one_a(1,3);R2_OOS_constr_two_a(1,3)];
Tab_5(5,4:6)=[WC_a(1,3) NaN NaN];
Tab_5(7,4:6)=[R2_OOS_a(1,5);R2_OOS_constr_one_a(1,5);R2_OOS_constr_two_a(1,5)];
Tab_5(8,4:6)=[WC_a(1,5) NaN NaN];

%-------------------------------------------------------------------------;
% Monte-Carlo simulations to verify statistical significance of the 
% main ROOS. 
%-------------------------------------------------------------------------;

%Obtain estimates for the full period: 1629-2015;
ret=[ret_1;ret_2;ret_34]; 
dg=[dg_1;dg_2;dg_34]; 
dp=[dp_1;dp_2;dp_34]; 

T=length(dp)-3+1;

%Rho;
mean_dp_org=mean(dp);
rho=exp(-mean_dp_org)/(1+exp(-mean_dp_org));

%Returns;
y=ret;
x=[ones(T-1,1) [dp_1(1:end-1);dp_2(1:end-1);dp_34(1:end-1)]];

[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
beta_ret_org=b(2,1); %Save for simulations;
res_ret=error;

%Dividend growth;
y=dg;
x=[ones(T-1,1) [dp_1(1:end-1);dp_2(1:end-1);dp_34(1:end-1)]];

[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
beta_dg_org=b(2,1); %Save for simulations;
res_dg=error;

%Dividend-to-price ratio;
y=[dp_1(2:end);dp_2(2:end);dp_34(2:end)];
x=[ones(T-1,1) [dp_1(1:end-1);dp_2(1:end-1);dp_34(1:end-1)]];

[b,b_std,tb,R2,AR2,error]=olsnw(y,x,1,1);
beta_dp_org=b(2,1); %Save for simulations;
res_dp=error;

%-----------------------------;
% Monte-Carlo simulations;
%-----------------------------;

%Null;
beta_ret=0;
beta_dp=beta_dp_org;
beta_dg=rho*beta_dp-1+beta_ret;

cov_dg_dp=cov(res_dg,res_dp);

To=T+200-1;     % Sample length (use 200 periods as burn-in)
No=1000;        % Number of simulated samples;

%To store simulation results;
R2_OOS_sim=zeros(No,5); 

for i=1:No

    %Joint distribution of errors;
    mu_error=zeros(To,2);
    error=mvnrnd(mu_error,cov_dg_dp);

    %Simulate returns, dividend growth, and the dividend-to-price ratio;
    ret=zeros(To,1);
    dg=zeros(To,1);
    dp=zeros(To,1);

    %Set initial value for dp;
    dp(1)=normrnd(0,std(res_dp)/(1-beta_dp));
        
    for t=2:To
        dp(t)=beta_dp*(dp(t-1))+error(t,2);
        dg(t)=beta_dg*(dp(t-1))+error(t,1);
        ret(t)=error(t,1)-rho*error(t,2);
    end;

    %Take out the burn-in period;
    ret=ret(201:end);
    dg=dg(201:end);
    dp=dp(200:end);

    %Out-of-sample predictions;
    a=72; %Starting point for expanding windows;

    for h=1:2:5 %For 1-,3- and 5-year forecasts;

        %Sum returns over h periods;    
        ret_h=filter(ones(h,1),1,ret);
        dp_h=dp(1:end-h); 

        %To store predictions;
        ret_f=zeros(T-h,1); %Forecasts using a constant and dp;
        ret_fc=zeros(T-h,1); %Forecasts based on a constant;

        for j=a:T-h
            [b,b_std,tb,R2,AR2,error]=olsnw(ret_h(h:j-1),[ones(j-h,1) dp_h(1:j-h)],0,0);
            ret_f(j+h-1)=b(1,1)+b(2,1)*dp_h(j);
            ret_fc(j+h-1)=mean(ret_h(h:j-1));
        end;

        %Actual returns and forecasts;
        ret_a=ret_h(a+h-1:end);
        ret_f=ret_f(a+h-1:end); 
        ret_fc=ret_fc(a+h-1:end);

        %Forecasting errors;
        err_f=ret_a-ret_f;
        err_fc=ret_a-ret_fc;

        %Out-of-sample R-square;
        R2_OOS_sim(i,h)=1.-(err_f'*err_f)./(err_fc'*err_fc);
    end;
end;

R2_OOS_sim_1y=R2_OOS_sim(:,1);
R2_OOS_sim_3y=R2_OOS_sim(:,3);
R2_OOS_sim_5y=R2_OOS_sim(:,5);

%Check;
mean(R2_OOS_sim_1y);
mean(R2_OOS_sim_3y);
mean(R2_OOS_sim_5y);

%P-values(R2_OOS): 1-year;
a=find(R2_OOS_sim_1y>R2_OOS_1y);
p_value_R2_OOS_1y=length(a)/length(R2_OOS_sim_1y);

%P-values(R2_OOS): 3-years;
a=find(R2_OOS_sim_3y>R2_OOS_3y);
p_value_R2_OOS_3y=length(a)/length(R2_OOS_sim_3y);

%P-values(R2_OOS): 5-years;
a=find(R2_OOS_sim_5y>R2_OOS_5y);
p_value_R2_OOS_5y=length(a)/length(R2_OOS_sim_5y);

%Collect results;
Tab_5(3,1:6)=[p_value_R2_OOS_1y ones(1,5)*NaN];
Tab_5(6,1:6)=[p_value_R2_OOS_3y ones(1,5)*NaN];
Tab_5(9,1:6)=[p_value_R2_OOS_5y ones(1,5)*NaN];

Tab_5





















